<?php namespace Myriad\Squire\Contracts;

use GuzzleHttp\Client as Guzzle;
use GuzzleHttp\Exception\ServerException;
use Myriad\Squire\SquireConfiguration;
use Myriad\Squire\SquireFile;
use Myriad\Squire\SquireHelpers;
use Myriad\Squire\SquireImage;

interface Squire {

  /**
   * Create a new instance of squire
   *
   * @param \Myriad\Squire\SquireConfiguration $config
   * @param \Myriad\Squire\SquireHelpers $helpers
   * @param \GuzzleHttp\Client $guzzle
   */
  public function __construct(SquireConfiguration $config, SquireHelpers $helpers, Guzzle $guzzle);

  /**
   * Get a file from Squire
   *
   * @param string $key the file key
   *
   * @return resource
   * @throws ServerException
   */
  public function get($key);

  /**
   * Put file to Squire
   *
   * @param resource|string|\SplFileInfo $resource
   * @param string $key the file key
   *
   * @throws ServerException
   */
  public function put($resource, $key);

  /**
   * Delete file from Squire
   *
   * @param string $key the file key
   *
   * @throws ServerException
   */
  public function delete($key);

  /**
   * Make a SquireImage
   *
   * @param string $key the file key
   *
   * @return SquireImage
   */
  public function image($key);

  /**
   * Make a SquireFile
   *
   * @param string $key the file key
   *
   * @return SquireFile
   */
  public function file($key);

  /**
   * Get the Squire configuration
   *
   * @return SquireConfiguration
   */
  public function getConfiguration();

  /**
   * Generate a unique key with a given extension
   *
   * @param string $extension The file extension
   * @param string $prefix A string to prefix the key with
   *
   * @return string
   */
  public function generateKey($extension, $prefix = '');

  /**
   * Generate a unique key from a file object
   *
   * @param \SplFileInfo $file The file object
   * @param string $prefix A string to prefix the key with
   *
   * @return string
   */
  public function generateKeyFromFile($file, $prefix = '');

}