<?php

use Mockery as m;

/**
 * Class SquireHelpersTest
 *
 * Test the SquireHelpers class
 */
class SquireHelpersTest extends SquireTestCase {

  /**
   * @var string the content of the temp file
   */
  private $tempContent = 'test content!';

  /**
   * Create a new temp file.
   *
   * @return string the filepath
   */
  private function createTempFile() {
    $temp = tempnam(sys_get_temp_dir(), 'squire');
    $resource = fopen($temp, 'r+');
    fwrite($resource, $this->tempContent);
    @fclose($resource);

    return $temp;
  }

  /**
   * Test the openResource method with an open resource
   */
  public function testOpenResourceWithOpenResource() {
    $temp = $this->createTempFile();
    $resource = fopen($temp, 'r+');

    // read to the end to simulate a freshly written file
    fread($resource, strlen($this->tempContent));
    $this->assertEquals(ftell($resource), strlen($this->tempContent), 'The resource pointer should be at the end of the file');

    $helpers = new \Myriad\Squire\SquireHelpers();
    $resource = $helpers->openResource($resource);
    $this->assertEquals(ftell($resource), 0, 'The get request body method should rewind the resource pointer');
    $this->assertResourceContent($resource);

    @fclose($resource);
    @unlink($temp);
  }

  /**
   * Test the openResource method with a closed resource
   */
  public function testOpenResourceWithClosedResource() {
    $temp = $this->createTempFile();
    $resource = fopen($temp, 'r+');
    fclose($resource);

    $this->setExpectedException('InvalidArgumentException', 'Resource stream is not open.');

    $helpers = new \Myriad\Squire\SquireHelpers();
    $resource = $helpers->openResource($resource);

    @fclose($resource);
    @unlink($temp);
  }

  /**
   * Test the openResource method with a SplFileInfo object.
   */
  public function testOpenResourceWithSplFileInfo() {
    $temp = $this->createTempFile();
    $file = new SplFileInfo($temp);

    $helpers = new \Myriad\Squire\SquireHelpers();
    $resource = $helpers->openResource($file);
    $this->assertResourceContent($resource);

    @fclose($resource);
    @unlink($temp);
  }

  /**
   * Test openResource method with a string filename.
   */
  public function testOpenResourceWithFilename() {
    $temp = $this->createTempFile();

    $helpers = new \Myriad\Squire\SquireHelpers();
    $resource = $helpers->openResource($temp);
    $this->assertResourceContent($resource);

    @fclose($resource);
    @unlink($temp);
  }

  /**
   * Test openResource method with a string filename.
   */
  public function testOpenResourceWithUnknownResource() {
    $resource = new stdClass();

    $helpers = new \Myriad\Squire\SquireHelpers();

    $this->assertEquals($resource, $helpers->openResource($resource));
  }

  /**
   * Assert the contents of the resource are correct.
   *
   * @param resource $resource The resource
   */
  private function assertResourceContent($resource) {
    $this->assertEquals($this->tempContent, fgets($resource));
  }

}